/*
 * Created by JFormDesigner on Sat Mar 07 06:21:42 EST 2009
 * This is a simple class which allows connection to a remote
 * database for loading resource records.
 *
 * The main purpose of this is so that a user can still access
 * collections from a database which is not running the same client
 * version.
 *
 */

package org.archiviststoolkit.plugin;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;
import java.sql.SQLException;
import javax.swing.*;
import com.jgoodies.forms.factories.*;
import com.jgoodies.forms.layout.*;
import org.archiviststoolkit.hibernate.SessionFactory;
import org.archiviststoolkit.model.Resources;
import org.archiviststoolkit.mydomain.LookupException;
import org.archiviststoolkit.mydomain.DomainObject;
import org.hibernate.cfg.Configuration;
import org.hibernate.*;
import org.hibernate.criterion.Order;

/**
 * @author Nathan Stevens
 */
public class RemoteDBConnectDialog extends JDialog {
    // the hibernate session facotory
    private org.hibernate.SessionFactory sessionFactory = null;

    // The session that is used through out to load records
    Session session = null;

    public RemoteDBConnectDialog(Frame owner) {
        super(owner, "Remote Database Connection");
        initComponents();
    }

    public RemoteDBConnectDialog(Dialog owner) {
        super(owner);
        initComponents();
    }

    /**
     * Just set this window invisible
     */
    private void okButtonActionPerformed() {
        setVisible(false);
    }

    /**
     * The connect button calls but all the work is done in connectToDatabase2.
     */
    private void connectToDatabase() {
        // disable the connect button and start the progress bar going
        button1.setEnabled(false);
        progressBar1.setIndeterminate(true);
        progressBar1.setStringPainted(true);
        progressBar1.setString("Connecting to database");

        Thread performer = new Thread(new Runnable() {
            public void run() {
                // try connecting to the database and if succesfull try loading the resource
                // records into the list
                if(connectToDatabase2()) {
                    progressBar1.setString("Loading records");
                    loadResourceRecords();
                }

                // all done, so enable the process button and stop the progress bar
                progressBar1.setIndeterminate(false);
                progressBar1.setStringPainted(false);
                button1.setEnabled(true);
            }
        });
        performer.start();
    }

    /**
     * Connect to the AT database at the given location. This does not check database version
     * so the it should be able to work with version 1.5 and 1.5.7
     */
    private boolean connectToDatabase2() {
        // based on the database type set the driver and hibernate dialect
        String databaseType = (String)comboBox2.getSelectedItem();
        String driverClass = "";
        String hibernateDialect = "";

        if (databaseType.equals(SessionFactory.DATABASE_TYPE_MYSQL)) {
			driverClass = "com.mysql.jdbc.Driver";
			hibernateDialect = "org.hibernate.dialect.MySQLInnoDBDialect";
		} else if (databaseType.equals(SessionFactory.DATABASE_TYPE_MICROSOFT_SQL_SERVER)) {
			driverClass = "com.microsoft.sqlserver.jdbc.SQLServerDriver";
			hibernateDialect = "org.hibernate.dialect.SQLServerDialect";
		} else if (databaseType.equals(SessionFactory.DATABASE_TYPE_ORACLE)) {
			driverClass = "oracle.jdbc.OracleDriver";
			hibernateDialect = "org.hibernate.dialect.Oracle10gDialect";
		} else { // should never get here
            System.out.println("Unknown database type : " + databaseType);
            return false;
        }

        // now attempt to build the session factory
        String databaseUrl = (String)comboBox1.getSelectedItem();
        String userName = textField1.getText();
        String password = passwordField1.getText();

        try {
            Configuration config = new Configuration().configure();
            Properties properties = config.getProperties();
            properties.setProperty("hibernate.connection.driver_class", driverClass);
            if (databaseType.equals(SessionFactory.DATABASE_TYPE_MYSQL)) {
                properties.setProperty("hibernate.connection.url", databaseUrl + "?useUnicode=yes&characterEncoding=utf8");
            } else {
                properties.setProperty("hibernate.connection.url", databaseUrl);
            }
            //deal with oracle specific settings
            if (databaseType.equals(SessionFactory.DATABASE_TYPE_ORACLE)) {
                properties.setProperty("hibernate.jdbc.batch_size", "0");
                properties.setProperty("hibernate.jdbc.use_streams_for_binary", "true");
                properties.setProperty("SetBigStringTryClob", "true");
            }
            properties.setProperty("hibernate.connection.username", userName);
            properties.setProperty("hibernate.connection.password", password);
            properties.setProperty("hibernate.dialect", hibernateDialect);
            config.setProperties(properties);
            sessionFactory = config.buildSessionFactory();

            //test the session factory to make sure it is working
            testHibernate();

            return true; // connected successfully so return true
        } catch (Exception hibernateException) {
            JOptionPane.showMessageDialog(this,
                    "Failed to start hibernate engine ...",
                    "Hibernate Error",
                    JOptionPane.ERROR_MESSAGE);
            return false;
        }
    }

    /**
     * Method to load all the resource records from the database
     */
    private void loadResourceRecords() {
        session = sessionFactory.openSession(); // open the session now
        
        final java.util.List completeList;
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			Criteria criteria = session.createCriteria(Resources.class);
			completeList = criteria.list();
			tx.commit();

		} catch (RuntimeException ex) {
			try {
				tx.rollback();
			} catch (HibernateException e) {
				JOptionPane.showMessageDialog(this,
                    "Failed to load resource records ...",
                    "Record Load Error",
                    JOptionPane.ERROR_MESSAGE);
                return;
			}
            return;
		}

        // add the records to the list in the UDT thread
		Runnable doWorkRunnable = new Runnable() {
            public void run() {
                // now sort the returned list
                Collections.sort(completeList);

                // clear any records in the list
                list1.removeAll();

                for (Object object : completeList) {
                    list1.addElement(object);
                }
            }
        };
        SwingUtilities.invokeLater(doWorkRunnable);
    }

    /**
     * Method to test the hibernate connection to make sure it can connect to an
     * AT database.
     *
     * @throws SQLException if it can't connect to the AT database
     */
    private void testHibernate() throws SQLException {
        Session testSession = sessionFactory.openSession();
        Query query = null;
        query = testSession.createQuery("select count(*) from Constants");
        Transaction tx = testSession.beginTransaction();
        query.list();
        tx.commit();
        testSession.close();
    }

    /**
     * function to return the selected record or null if nothing is selected
     */
    public DomainObject getCurrentRecord() {
        Object object  = list1.getSelectedValue();
        if(object != null) {
            return (DomainObject)object;
        } else {
            return null;
        }
    }

    private void initComponents() {
        // JFormDesigner - Component initialization - DO NOT MODIFY  //GEN-BEGIN:initComponents
        // Generated using JFormDesigner non-commercial license
        dialogPane = new JPanel();
        contentPanel = new JPanel();
        label1 = new JLabel();
        comboBox1 = new JComboBox();
        label2 = new JLabel();
        textField1 = new JTextField();
        label3 = new JLabel();
        passwordField1 = new JPasswordField();
        label4 = new JLabel();
        comboBox2 = new JComboBox();
        label6 = new JLabel();
        textField2 = new JTextField();
        scrollPane1 = new JScrollPane();
        list1 = new FilteringJList();
        label5 = new JLabel();
        progressBar1 = new JProgressBar();
        buttonBar = new JPanel();
        button1 = new JButton();
        okButton = new JButton();
        CellConstraints cc = new CellConstraints();

        //======== this ========
        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());

        //======== dialogPane ========
        {
            dialogPane.setBorder(Borders.DIALOG_BORDER);
            dialogPane.setLayout(new BorderLayout());

            //======== contentPanel ========
            {
                contentPanel.setLayout(new FormLayout(
                    new ColumnSpec[] {
                        FormFactory.DEFAULT_COLSPEC,
                        FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
                        new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW)
                    },
                    new RowSpec[] {
                        FormFactory.DEFAULT_ROWSPEC,
                        FormFactory.LINE_GAP_ROWSPEC,
                        FormFactory.DEFAULT_ROWSPEC,
                        FormFactory.LINE_GAP_ROWSPEC,
                        FormFactory.DEFAULT_ROWSPEC,
                        FormFactory.LINE_GAP_ROWSPEC,
                        FormFactory.DEFAULT_ROWSPEC,
                        FormFactory.LINE_GAP_ROWSPEC,
                        FormFactory.DEFAULT_ROWSPEC,
                        FormFactory.LINE_GAP_ROWSPEC,
                        FormFactory.DEFAULT_ROWSPEC,
                        FormFactory.LINE_GAP_ROWSPEC,
                        FormFactory.DEFAULT_ROWSPEC
                    }));

                //---- label1 ----
                label1.setText("Connection URL");
                contentPanel.add(label1, cc.xy(1, 1));

                //---- comboBox1 ----
                comboBox1.setEditable(true);
                contentPanel.add(comboBox1, cc.xy(3, 1));

                //---- label2 ----
                label2.setText("Username");
                contentPanel.add(label2, cc.xy(1, 3));

                //---- textField1 ----
                textField1.setColumns(25);
                contentPanel.add(textField1, cc.xy(3, 3));

                //---- label3 ----
                label3.setText("Password");
                contentPanel.add(label3, cc.xy(1, 5));
                contentPanel.add(passwordField1, cc.xy(3, 5));

                //---- label4 ----
                label4.setText("Database Type");
                contentPanel.add(label4, cc.xy(1, 7));

                //---- comboBox2 ----
                comboBox2.setModel(new DefaultComboBoxModel(new String[] {
                    "MySQL",
                    "Oracle",
                    "Microsoft SQL Server"
                }));
                contentPanel.add(comboBox2, cc.xy(3, 7));

                //---- label6 ----
                label6.setText("Filter Results");
                contentPanel.add(label6, cc.xy(1, 9));
                contentPanel.add(textField2, cc.xy(3, 9));

                //======== scrollPane1 ========
                {

                    //---- list1 ----
                    list1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                    list1.setVisibleRowCount(15);
                    list1.installJTextField(textField2);
                    scrollPane1.setViewportView(list1);
                }
                contentPanel.add(scrollPane1, cc.xywh(1, 11, 3, 1));

                //---- label5 ----
                label5.setText("Progress");
                contentPanel.add(label5, cc.xy(1, 13));
                contentPanel.add(progressBar1, cc.xy(3, 13));
            }
            dialogPane.add(contentPanel, BorderLayout.CENTER);

            //======== buttonBar ========
            {
                buttonBar.setBorder(Borders.BUTTON_BAR_GAP_BORDER);
                buttonBar.setLayout(new FormLayout(
                    new ColumnSpec[] {
                        FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
                        FormFactory.DEFAULT_COLSPEC,
                        FormFactory.GLUE_COLSPEC,
                        FormFactory.DEFAULT_COLSPEC,
                        FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
                        FormFactory.BUTTON_COLSPEC
                    },
                    RowSpec.decodeSpecs("pref")));

                //---- button1 ----
                button1.setText("Connect");
                button1.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        connectToDatabase();
                    }
                });
                buttonBar.add(button1, cc.xy(4, 1));

                //---- okButton ----
                okButton.setText("OK");
                okButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        okButtonActionPerformed();
                    }
                });
                buttonBar.add(okButton, cc.xy(6, 1));
            }
            dialogPane.add(buttonBar, BorderLayout.SOUTH);
        }
        contentPane.add(dialogPane, BorderLayout.CENTER);
        pack();
        setLocationRelativeTo(getOwner());
        // JFormDesigner - End of component initialization  //GEN-END:initComponents
    }

    // JFormDesigner - Variables declaration - DO NOT MODIFY  //GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    private JPanel dialogPane;
    private JPanel contentPanel;
    private JLabel label1;
    private JComboBox comboBox1;
    private JLabel label2;
    private JTextField textField1;
    private JLabel label3;
    private JPasswordField passwordField1;
    private JLabel label4;
    private JComboBox comboBox2;
    private JLabel label6;
    private JTextField textField2;
    private JScrollPane scrollPane1;
    private FilteringJList list1;
    private JLabel label5;
    private JProgressBar progressBar1;
    private JPanel buttonBar;
    private JButton button1;
    private JButton okButton;
    // JFormDesigner - End of variables declaration  //GEN-END:variables
}
