/*
 * Created by JFormDesigner on Fri Mar 06 13:15:39 EST 2009
 */

package org.archiviststoolkit.plugin;

import java.awt.*;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.*;
import java.io.File;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import javax.swing.*;
import javax.swing.border.*;
import com.jgoodies.forms.factories.*;
import com.jgoodies.forms.layout.*;
import org.archiviststoolkit.swing.*;
import org.archiviststoolkit.ApplicationFrame;
import org.archiviststoolkit.util.StringHelper;
import org.archiviststoolkit.dialog.ATFileChooser;
import org.archiviststoolkit.mydomain.DomainTableWorkSurface;
import org.archiviststoolkit.mydomain.DomainObject;
import org.archiviststoolkit.model.Resources;
import org.archiviststoolkit.model.ResourcesComponents;

/**
 * @author Nathan Stevens
 */
public class ShootListPluginFrame extends JFrame implements ClipboardOwner {
    private ApplicationFrame mainFrame; // the main AT application frame
    private DomainTableWorkSurface worksurface; // the currently selected worksurface
    private DomainObject record; // the current domain object
    private RemoteDBConnectDialog remoteDB; // this setup a connection to a remote AT database
    private int componentCount = 0; // keep track of the components processed

    /**
     * The default constructor
     */
    public ShootListPluginFrame() {
        super("ShootList v1.0");
        initComponents();
    }

    /**
     * save the information displayed in the text Area to a text file
     */
    private void saveInfoToFile() {
        ATFileChooser filechooser = new ATFileChooser();
        if (filechooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
            File selectedFile = filechooser.getSelectedFile();

            try {
                FileWriter fileWriter = new FileWriter(selectedFile);
                BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

                // get the text to write out
                bufferedWriter.write(textArea1.getText());

                bufferedWriter.close();

            } catch (IOException ioe) {
                JOptionPane.showMessageDialog(this,
                    "Error saving shoot list to text file ...",
                    "Error Saving File",
                    JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    // just close the window and dispose it
    private void okButtonActionPerformed() {
        setVisible(false);
        dispose();
    }

    // Method to copy the contents of the textArea to the system clipboard
    private void copyToClipboard() {
        StringSelection stringSelection = new StringSelection(textArea1.getText());
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        clipboard.setContents(stringSelection, this);
    }

    /**
     * Method to load the currently selected record in the AT. This is done
     * in a thread
     */
    private boolean loadCurrentRecord() {
        // alert the user that the record is now being loaded
        textField1.setText("Record : loading ...");

        DomainObject newRecord;

        // check to see if we are loading a record from an other AT database than the
        // one the client is connected to
        if(remoteDB != null && remoteDB.isVisible()) {
            newRecord = remoteDB.getCurrentRecord();
        } else if(worksurface != null){ // trying loading from the list displayed in AT client
            newRecord = worksurface.getCurrentDomainObjectFromDatabase();
        } else {
            newRecord = null;
        }

        // check to make sure record is an instnace of Resources
        if(newRecord != null && newRecord instanceof Resources) {
            record = newRecord;
            return true;
        } else {
            // alert user thet there is a problem
            JOptionPane.showMessageDialog(this,
                    "Error Loading Selected Record ...",
                    "Error Loading Record",
                    JOptionPane.ERROR_MESSAGE);

            textField1.setText("Record : ");
            record = null;
            return false;
        }
    }

    /**
     * Method to traverse the DO in this record and extract the relevant fields
     */
    private void processRecord() {
        // process the record now
        textArea1.setText(comboBox1.getSelectedItem() + " \t " + comboBox2.getSelectedItem() + "\n");

        // disable the process button and start the progress bar going
        button4.setEnabled(false);
        progressBar1.setIndeterminate(true);

        // set the number of components processed to zero
        componentCount = 0;

        // start the processing the components in a thread now
        Thread performer = new Thread(new Runnable() {
            public void run() {
                if(!loadCurrentRecord()) { // no record to process so return
                    progressBar1.setIndeterminate(false);
                    button4.setEnabled(true);
                    return;
                }

                Resources resource = (Resources) record;

                // set the name of the record
                textField1.setText("Record : " + StringHelper.tagRemover(resource.getTitle()));

                // load the resource components. should do this in thread but doesn't matter
                for (ResourcesComponents component : resource.getResourcesComponents()) {
                    processComponents(component);
                }

                // all done, so enable the process button and stop the progress bar
                progressBar1.setIndeterminate(false);
                button4.setEnabled(true);
            }
        });
        performer.start();
    }

    /**
     * Method to process all the components in the record. This make recursive calles
     *
     * @param component The Resource component to process
     */
    private void processComponents(ResourcesComponents component) {
        // only want to process item or file level components so skip those with
        // children
        if (component.isHasChild()) {
			for (ResourcesComponents childComponent : component.getResourcesComponents()) {
				processComponents(childComponent);
			}
		}

        // check the level of this component so that we only process item and file level
        // components
        String level = component.getLevel();
        if(level.equals("item") || level.equals("file")) {
            // process the component now
            componentCount++; // increment the component count now

            String title = getTitle(component.getTitle());
            String persistentID = component.getPersistentId();

            // based on the selection put the correct information in the correct
            // column. Should reallly thing of a scalable way to do this
            if (comboBox1.getSelectedIndex() == 0) {
                textArea1.append(title + " \t ");
            } else { // the persistent id goes here
                textArea1.append(persistentID + " \t ");
            }

            if (comboBox2.getSelectedIndex() == 0) {
                textArea1.append(title + "\n");
            } else { // the persistent id goes here
                textArea1.append(persistentID + "\n");
            }

            // update the label now
            label3.setText("" + componentCount);
        }
    }

    /**
     * Method to return the title only from the full title
     * This is needed because some records have a bunch of other stuff
     * in the title than just the title.
     *
     * @param fullTitle The title with everything else in there
     * @return Just the title and number
     */
    private String getTitle(String fullTitle) {
        String title = "";

        // put some spaces around any tags
        String newFullTitle1 = fullTitle.replaceAll("<", " <");
        String newFullTitle2 = newFullTitle1.replaceAll(">", " >");

        // remove the tags now
        title = StringHelper.tagRemover(newFullTitle2);

        // remove the leading space around ":" character so
        // that it makes it clear what label and the real text
        return title.replaceAll(" :", ":");
    }

    private void button3ActionPerformed() {
        // open the db connection wizard
        if(remoteDB == null) {
            remoteDB = new RemoteDBConnectDialog(this);
        }

        remoteDB.setVisible(true);
    }

    private void initComponents() {
        // JFormDesigner - Component initialization - DO NOT MODIFY  //GEN-BEGIN:initComponents
        // Generated using JFormDesigner non-commercial license
        dialogPane = new JPanel();
        contentPanel = new JPanel();
        textField1 = new JTextField();
        button4 = new JButton();
        label2 = new JLabel();
        label3 = new JLabel();
        panel1 = new JPanel();
        comboBox1 = new JComboBox();
        comboBox2 = new JComboBox();
        scrollPane1 = new JScrollPane();
        textArea1 = new PrintableJTextArea();
        label1 = new JLabel();
        progressBar1 = new JProgressBar();
        buttonBar = new JPanel();
        button3 = new JButton();
        button1 = new JButton();
        button2 = new JButton();
        okButton = new JButton();
        CellConstraints cc = new CellConstraints();

        //======== this ========
        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());

        //======== dialogPane ========
        {
            dialogPane.setBorder(new EmptyBorder(12, 12, 12, 12));
            dialogPane.setLayout(new BorderLayout());

            //======== contentPanel ========
            {
                contentPanel.setLayout(new FormLayout(
                    "default, default, default:grow",
                    "default, default, default, default, fill:default:grow"));

                //---- textField1 ----
                textField1.setText("Record :");
                textField1.setEditable(false);
                contentPanel.add(textField1, cc.xywh(1, 1, 3, 1));

                //---- button4 ----
                button4.setText("Process");
                button4.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        processRecord();
                    }
                });
                contentPanel.add(button4, cc.xy(1, 2));

                //---- label2 ----
                label2.setText("# Processed: ");
                contentPanel.add(label2, cc.xy(2, 2));

                //---- label3 ----
                label3.setText("0");
                label3.setForeground(Color.blue);
                contentPanel.add(label3, cc.xy(3, 2));

                //======== panel1 ========
                {
                    panel1.setLayout(new FormLayout(
                        new ColumnSpec[] {
                            FormFactory.DEFAULT_COLSPEC,
                            FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
                            new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, FormSpec.DEFAULT_GROW),
                            FormFactory.LABEL_COMPONENT_GAP_COLSPEC,
                            FormFactory.DEFAULT_COLSPEC
                        },
                        RowSpec.decodeSpecs("default")));

                    //---- comboBox1 ----
                    comboBox1.setModel(new DefaultComboBoxModel(new String[] {
                        "Title",
                        "PersistentID"
                    }));
                    panel1.add(comboBox1, cc.xy(1, 1));

                    //---- comboBox2 ----
                    comboBox2.setModel(new DefaultComboBoxModel(new String[] {
                        "Title",
                        "PersistentID"
                    }));
                    comboBox2.setSelectedIndex(1);
                    panel1.add(comboBox2, cc.xy(5, 1));
                }
                contentPanel.add(panel1, cc.xywh(1, 3, 3, 1));

                //======== scrollPane1 ========
                {

                    //---- textArea1 ----
                    textArea1.setRows(20);
                    textArea1.setEditable(false);
                    scrollPane1.setViewportView(textArea1);
                }
                contentPanel.add(scrollPane1, cc.xywh(1, 4, 3, 1));

                //---- label1 ----
                label1.setText("Progress");
                contentPanel.add(label1, cc.xy(1, 5));
                contentPanel.add(progressBar1, cc.xywh(2, 5, 2, 1));
            }
            dialogPane.add(contentPanel, BorderLayout.CENTER);

            //======== buttonBar ========
            {
                buttonBar.setBorder(new EmptyBorder(12, 0, 0, 0));
                buttonBar.setLayout(new GridBagLayout());
                ((GridBagLayout)buttonBar.getLayout()).columnWidths = new int[] {0, 0, 0, 80};
                ((GridBagLayout)buttonBar.getLayout()).columnWeights = new double[] {1.0, 0.0, 0.0, 0.0};

                //---- button3 ----
                button3.setText("DB");
                button3.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        button3ActionPerformed();
                    }
                });
                buttonBar.add(button3, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                    new Insets(0, 0, 0, 5), 0, 0));

                //---- button1 ----
                button1.setText("Save");
                button1.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        saveInfoToFile();
                    }
                });
                buttonBar.add(button1, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                    new Insets(0, 0, 0, 5), 0, 0));

                //---- button2 ----
                button2.setText("Copy");
                button2.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        copyToClipboard();
                    }
                });
                buttonBar.add(button2, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                    new Insets(0, 0, 0, 5), 0, 0));

                //---- okButton ----
                okButton.setText("OK");
                okButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        okButtonActionPerformed();
                    }
                });
                buttonBar.add(okButton, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                    new Insets(0, 0, 0, 0), 0, 0));
            }
            dialogPane.add(buttonBar, BorderLayout.SOUTH);
        }
        contentPane.add(dialogPane, BorderLayout.CENTER);
        pack();
        setLocationRelativeTo(getOwner());
        // JFormDesigner - End of component initialization  //GEN-END:initComponents
    }

    // JFormDesigner - Variables declaration - DO NOT MODIFY  //GEN-BEGIN:variables
    // Generated using JFormDesigner non-commercial license
    private JPanel dialogPane;
    private JPanel contentPanel;
    private JTextField textField1;
    private JButton button4;
    private JLabel label2;
    private JLabel label3;
    private JPanel panel1;
    private JComboBox comboBox1;
    private JComboBox comboBox2;
    private JScrollPane scrollPane1;
    private PrintableJTextArea textArea1;
    private JLabel label1;
    private JProgressBar progressBar1;
    private JPanel buttonBar;
    private JButton button3;
    private JButton button1;
    private JButton button2;
    private JButton okButton;
    // JFormDesigner - End of variables declaration  //GEN-END:variables

    // method to set the main AT application frame
    public void setApplicationFrame(ApplicationFrame mainFrame) {
        this.mainFrame = mainFrame;
        if(mainFrame != null) {
            worksurface = mainFrame.getWorkSurfaceContainer().getCurrentWorkSurface();
        }
    }

   /**
   * Empty implementation of the ClipboardOwner interface.
   */
   public void lostOwnership(Clipboard aClipboard, Transferable aContents) {
        //do nothing
   }
}
